from setuptools import setup

setup(
    name="TwitchPy",
    version="0.1",
    description="This is a package you can use to connect with the Twitch API, manage a channel, create bots, etc",
    author="DaCasBe",
    author_email="dacasbe97@gmail.com",
    url="https://github.com/DaCasBe/TwitchPy",
    packages=["twitchpy"]
)